<?php

namespace App\Factory;

class ConsumerFactory {
    
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getConsumerByEmail($email, $standardClient) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
			       NMCONSUMIDOR, NRCPFRESPCON,
			       CDTIPOCONS

		      FROM CONSUMIDOR

		     WHERE CDCLIENTE          = :CDCLIENTE 
		       AND LOWER(DSEMAILCONS) = LOWER(:DSEMAILCONS)";
		    
        $params = [
            ":CDCLIENTE"    => $standardClient,
            ":DSEMAILCONS"  => $email
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getConsumerByPhone($phone, $standardClient) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
		       NMCONSUMIDOR, NRCPFRESPCON,
		       CDTIPOCONS
	 		   
            FROM CONSUMIDOR

            WHERE (NRTELECONS = :NRTELECONS
                OR NRTELE2CONS = :NRCELULARCONS
                OR NRCELULARCONS = :NRCELULARCONS)
              AND CDCLIENTE      = :CDCLIENTE";
        
        $params = [
            ":NRTELECONS"    => $phone,
            ":NRCELULARCONS" => $phone,
            ":CDCLIENTE"     => $standardClient
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getConsumerByCPF($cpf, $standardClient) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
			       NMCONSUMIDOR, NRCPFRESPCON,
			       CDTIPOCONS

		      FROM CONSUMIDOR

		     WHERE CDCLIENTE          = :CDCLIENTE 
		       AND NRCPFRESPCON = :NRCPFRESPCON";
		    
        $params = [
            ":CDCLIENTE"    => $standardClient,
            ":NRCPFRESPCON"  => $cpf
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getConsumerById($cdCliente, $cdConsumidor) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
			       NMCONSUMIDOR, NRCPFRESPCON,
			       CDTIPOCONS

		    FROM CONSUMIDOR

		    WHERE CDCLIENTE    = :CDCLIENTE 
		      AND CDCONSUMIDOR = :CDCONSUMIDOR";
		    
		$params = [
		    ":CDCLIENTE"    => $cdCliente,
            ":CDCONSUMIDOR" => $cdConsumidor
		];
		
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getConsumerByIdConsumer($cdIdConsumid) {
        $sql = "
            SELECT CDCLIENTE, CDCONSUMIDOR,
			       NMCONSUMIDOR, NRCPFRESPCON,
			       CDTIPOCONS

		    FROM CONSUMIDOR

		    WHERE CDIDCONSUMID    = :CDIDCONSUMID"; 
		    
		$params = [
		    ":CDIDCONSUMID"    => $cdIdConsumid
		];
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getNeighborhoodCode ($consumerDetails) {
        $sql = "
            SELECT CDBAIRRO

              FROM BAIRRO

             WHERE CDPAIS          = :CDPAIS
               AND SGESTADO        = :SGESTADO
               AND CDMUNICIPIO     = :CDMUNICIPIO
               AND UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)";
        
        $params = [
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => $consumerDetails["SGESTADO"],
            ":CDMUNICIPIO" => $consumerDetails["CDMUNICIPIO"],
            ":NMBAIRRO"    => $consumerDetails["neighborhood"]
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result)) return null;
        
        return $result[0]["CDBAIRRO"];
    }
    
    public function getCityCode ($uf, $city) {
        $sql = "
            SELECT CDMUNICIPIO 
    	    FROM MUNICIPIO
    	    WHERE SGESTADO = :SGESTADO
             AND CDPAIS   = '0055'
    		 AND NVL((SELECT CDMUNICIPIO
    		            FROM MUNICIPIO
    		           WHERE UPPER(NMMUNICIPIO) = :NMMUNICIPIO
    		             AND ROWNUM = 1),
    		         (SELECT CDMUNICIPIO
    		            FROM MUNICIPIO
    		           WHERE UPPER(NMMUNICIPIO) LIKE '%'||:NMMUNICIPIO||'%'
    		             AND ROWNUM = 1)) = CDMUNICIPIO";
    		             
    	$params = [
    	    ":SGESTADO"    => strtoupper($uf),
    	    ":NMMUNICIPIO" => strtoupper($this->removeAcentos($city))
    	];
    	
    	$result = $this->connection->fetchAll($sql, $params);
    	
    	if (empty($result)) return null;
    	
    	return $result[0]["CDMUNICIPIO"];
    }
    
    public function registerConsumer ($consumerDetails) {
        $sql = "
            INSERT INTO CONSUMIDOR (
                CDCLIENTE, CDCONSUMIDOR, NMCONSUMIDOR, IDSITCONSUMI,
                CDCCUSCLIE, IDCONSUMIDOR, IDATUCONSUMI, IDTPVENDACONS,
                IDIMPCPFCUPOM, IDCADCONFLIBCON, IDPERCONSPRODEX, IDTPSELMANHA,
                IDTPSEALMOCO, IDTPSELTARDE, IDCRACHAMESTRE, CDPAIS,
                SGESTADO, CDMUNICIPIO, CDTIPOCONS, CDIDCONSUMID,
                DSENDECONS, NMBAIRCONS, NRCEPCONS, NRTELECONS,
                DSEMAILCONS, NRENDECONS, NMRESPCONS, DSREFERENDECONS, DSINFOCONS, NRCPFRESPCON, DSCOMPLENDECONS
            )
            VALUES (
                :CDCLIENTE, :CDCONSUMIDOR, :NMCONSUMIDOR, :IDSITCONSUMI,
                :CDCCUSCLIE, :IDCONSUMIDOR, :IDATUCONSUMI, :IDTPVENDACONS,
                :IDIMPCPFCUPOM, :IDCADCONFLIBCON, :IDPERCONSPRODEX, :IDTPSELMANHA,
                :IDTPSEALMOCO, :IDTPSELTARDE, :IDCRACHAMESTRE, :CDPAIS,
                :SGESTADO, :CDMUNICIPIO, :CDTIPOCONS, :CDIDCONSUMID,
                :DSENDECONS, :NMBAIRCONS, :NRCEPCONS, :NRTELECONS,
                :DSEMAILCONS, :NRENDECONS, :NMRESPCONS, :DSREFERENDECONS, :DSINFOCONS, :NRCPFRESPCON, :DSCOMPLENDECONS
            )";
        
        $params = [
            ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
            ":NMCONSUMIDOR"    => substr($consumerDetails["name"], 0, 50),
            ":IDSITCONSUMI"    => "1",
            ":CDCCUSCLIE"      => $consumerDetails["CDCCUSCLIE"],
            ":IDCONSUMIDOR"    => "C",
            ":IDATUCONSUMI"    => "S",
            ":IDTPVENDACONS"   => "7",
            ":IDIMPCPFCUPOM"   => "S",
            ":IDCADCONFLIBCON" => "N",
            ":IDPERCONSPRODEX" => "S",
            ":IDTPSELMANHA"    => "N", 
            ":IDTPSEALMOCO"    => "N", 
            ":IDTPSELTARDE"    => "N", 
            ":IDCRACHAMESTRE"  => "N", 
            ":CDPAIS"          => "0055",
            ":SGESTADO"        => $consumerDetails["SGESTADO"],
            ":CDMUNICIPIO"     => $consumerDetails["CDMUNICIPIO"],
            ":CDTIPOCONS"      => $consumerDetails["CDTIPOCONS"],
            ":CDIDCONSUMID"    => $consumerDetails["CDCONSUMIDOR"],
            ":DSENDECONS"      => substr($consumerDetails["street"], 0, 50),
            ":NMBAIRCONS"      => substr($consumerDetails["neighborhood"], 0, 30),
            ":NRCEPCONS"       => preg_replace("/[^0-9]/", "", $consumerDetails["cep"]),
            ":NRTELECONS"      => !empty($consumerDetails["phone"]) ? preg_replace("/[^0-9]/", "", $consumerDetails["phone"]) : null,
            ":DSEMAILCONS"     => !empty($consumerDetails["email"]) ? $consumerDetails["email"] : null,
            ":NRENDECONS"       => $consumerDetails["number"],
            ":NMRESPCONS"      => substr($consumerDetails["name"], 0, 50),
            ":DSREFERENDECONS" => !empty($consumerDetails["referencePoint"]) ? $consumerDetails["referencePoint"] : null,
            ":DSINFOCONS"      => "SDE - Incluido",
            ":NRCPFRESPCON"    => $consumerDetails["cpf"],
            ":DSCOMPLENDECONS" => !empty($consumerDetails["complement"]) ? $consumerDetails["complement"] : null
        ];
        
        
        $this->connection->executeQuery($sql, $params);
        
        $sql = "
            SELECT *
              FROM CONSUMIDOR
             WHERE CDCLIENTE    = :CDCLIENTE
               AND CDCONSUMIDOR = :CDCONSUMIDOR";
            
        $params = [
            ":CDCLIENTE"    => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumerDetails["CDCONSUMIDOR"]
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getIdConsumerByEmail($refConsumer){
        $sql = "
            SELECT C.CDCLIENTE, C.CDCONSUMIDOR, C.CDIDCONSUMID, 
                   C.NMCONSUMIDOR, C.NRCPFRESPCON, C.CDTIPOCONS,
                   C.DSEMAILCONS, C.NRTELECONS, C.NRCELULARCONS
             FROM CONSUMIDOR C
               WHERE C.DSEMAILCONS = :DSEMAILCONS
        ";
        
        $params = [
            ":DSEMAILCONS" => $refConsumer
            ];
            
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getIdConsumerByCpf($refConsumer){
        $sql = "
            SELECT C.CDCLIENTE, C.CDCONSUMIDOR, C.CDIDCONSUMID, 
                   C.NMCONSUMIDOR, C.NRCPFRESPCON, C.CDTIPOCONS,
                   C.DSEMAILCONS, C.NRTELECONS, C.NRCELULARCONS
             FROM CONSUMIDOR C
               WHERE C.NRCPFRESPCON = :NRCPFRESPCON
        ";
        
        $params = [
            ":NRCPFRESPCON" => $refConsumer
            ];
            
        return $this->connection->fetchAll($sql, $params);
    }
    
        public function getIdConsumerByRef($refConsumer){
        $sql = "
            SELECT C.CDCLIENTE, C.CDCONSUMIDOR, C.CDIDCONSUMID, 
                   C.NMCONSUMIDOR, C.NRCPFRESPCON, C.CDTIPOCONS,
                   C.DSEMAILCONS, C.NRTELECONS, C.NRCELULARCONS
             FROM CONSUMIDOR C
               WHERE C.CDIDCONSUMID = :CDIDCONSUMID
        ";
        
        $params = [
            ":CDIDCONSUMID" => $refConsumer
            ];
            
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }

    
}
